package oracle.iam.assets.eventhandlers.extuserinfo.service;


import Thor.API.Exceptions.tcAPIException;
import Thor.API.Exceptions.tcColumnNotFoundException;
import Thor.API.Exceptions.tcITResourceDefinitionNotFoundException;
import Thor.API.Exceptions.tcITResourceNotFoundException;
import Thor.API.Exceptions.tcInvalidLookupException;
import Thor.API.Operations.tcITResourceDefinitionOperationsIntf;
import Thor.API.Operations.tcITResourceInstanceOperationsIntf;
import Thor.API.Operations.tcLookupOperationsIntf;
import Thor.API.tcResultSet;

import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import oracle.iam.assets.eventhandlers.exception.ContractorIDPPEventHandlerException;
import oracle.iam.assets.eventhandlers.extuserinfo.utils.common.Constants;
import oracle.iam.platform.OIMClient;
import oracle.iam.platform.Platform;


/**
 *  This class is for getting and setting the input and connection parameters from OIM.
 *
 */

public class OIMService {

    private tcITResourceInstanceOperationsIntf oimITResInstUtility;
    private tcITResourceDefinitionOperationsIntf oimITResDefUtility;
    private tcLookupOperationsIntf lookupOperations;

    private Hashtable<String, String> connParams =
        new Hashtable<String, String>();
    private Hashtable<String, String> inputParams =
        new Hashtable<String, String>();
    private OIMClient oimClient = null;

    public OIMService() {
        super();
    }


    /**
     * The method used for getting connection parameters for Contractor DB.
     * @param itResName      : IT resource Name declared in OIM.
     * @param itResourceType : IT resource type declared in OIM
     * @return connParams    : HashTable that contains all the parameters that
     *               are required to establish the connection with the service.
     */

    public Hashtable<String, String> getConnParameters(String itResName,
                                                       String itResourceType) throws tcITResourceNotFoundException,
                                                                                     tcAPIException,
                                                                                     tcAPIException,
                                                                                     tcColumnNotFoundException,
                                                                                     ContractorIDPPEventHandlerException {
        tcResultSet resultSet = null;
        long itResKey;
        String oimITResourceParameterName;
        String oimITResourceParameterValue;

        //        Hashtable env = new Hashtable();
        oimITResDefUtility =
                Platform.getService(tcITResourceDefinitionOperationsIntf.class);

        oimITResInstUtility =
                Platform.getService(tcITResourceInstanceOperationsIntf.class);

        itResKey = fetchITResourceKey(itResName, itResourceType);
        resultSet =
                oimITResInstUtility.getITResourceInstanceParameters(itResKey);
        LOGGER("Enter OIMService#getConnParameters()");
        if (resultSet != null && !resultSet.isEmpty()) {
            for (int i = 0; i < resultSet.getRowCount(); i++) {
                resultSet.goToRow(i);
                oimITResourceParameterName =
                        resultSet.getStringValue(Constants.OIM_ITRESOURCE_TYPE_PARAMETER_NAME);
                oimITResourceParameterValue =
                        resultSet.getStringValue(Constants.OIM_ITRESOURCE_PARAMETER_VALUE);
                LOGGER(" Resource Name : " + oimITResourceParameterName +
                       "  Value " + oimITResourceParameterValue);
                connParams.put(oimITResourceParameterName,
                               oimITResourceParameterValue);
            }
        }
        LOGGER("Exit OIMService#getConnParameters()");
        return connParams;
    }

    /**
     * This method is used for getting the inputParameters that are required to from
     *    the query statement from Lookup Definition.
     * @param psLookupCode  : LookUp code value
     * @return inputParams  : HashTable which contains the input parameters used to form the query statement.
     * @throws tcAPIException
     * @throws tcColumnNotFoundException
     * @throws ContractorIDPPEventHandlerException
     */
    public Hashtable<String, String> getinputParameters(String psLookupCode) throws tcAPIException,
                                                                                    tcColumnNotFoundException,
                                                                                    ContractorIDPPEventHandlerException {
        String codeKey;
        String decode;
        lookupOperations = Platform.getService(tcLookupOperationsIntf.class);
        tcResultSet lookupResultSet = getLookupCodeInfo(psLookupCode);
        LOGGER("Enter OIMService#getinputParameters()");
        if (lookupResultSet != null && !lookupResultSet.isEmpty()) {
            for (int i = 0; i < lookupResultSet.getRowCount(); i++) {
                lookupResultSet.goToRow(i);
                codeKey =
                        lookupResultSet.getStringValue(Constants.OIM_LOOKUP_CODE_KEY);
                decode =
                        lookupResultSet.getStringValue(Constants.OIM_LOOKUP_DECODE_VALUE);
                LOGGER("Code Key : " + codeKey + "Decode Value :  " + decode);
                inputParams.put(codeKey, decode);
            }
        }
        LOGGER("Exit OIMService#getinputParameters()");
        return inputParams;
    }


    /**
     * This method is used to fetch the key associated with the IT Resource.
     * @param itResName       : IT Resource name whose key needs to be fetched.
     * @param itResourceType  : IT Resource Type.
     * @return itResKey       : key associated with the IT Resources.
     * @throws ContractorIDPPEventHandlerException
     */
    public long fetchITResourceKey(String itResName,
                                   String itResourceType) throws ContractorIDPPEventHandlerException {
        long itResKey = -1;
        try {
            long itResourceTypeKey = getITResourceTypeKey(itResourceType);

            tcResultSet resultSet =
                oimITResInstUtility.findITResourceInstancesByDefinition(itResourceTypeKey);
            if (resultSet != null && !resultSet.isEmpty()) {
                int i = resultSet.getRowCount();
                int j = 0;
                while (j < i) {
                    resultSet.goToRow(j);
                    j++;
                    String itResourceName =
                        resultSet.getStringValue(Constants.IT_RESOURCE_NAME);
                    if (itResourceName.equals(itResName)) {
                        itResKey =
                                resultSet.getLongValue(Constants.IT_RESOURCE_KEY);
                    } else {
                        continue;
                    }
                }
            }
        } catch (tcColumnNotFoundException ex) {
            String reason =
                "tcColumnNotFoundException occurred when fetching key of the IT resource " +
                itResName;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (tcITResourceDefinitionNotFoundException ex) {
            String reason =
                "tcITResourceDefinitionNotFoundException occurred when fetching key of the IT resource " +
                itResName;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (tcAPIException ex) {
            String reason =
                "tcAPIException occurred when fetching key of the IT resource " +
                itResName;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (Exception ex) {
            String reason =
                "Exception occurred when fetching key of the IT resource " +
                itResName;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        }
        return itResKey;
    }


    /**
     * This method is used for getting the key associated with the given IT Resource Type.
     * @param itResourceType  : Type of the IT Resource whose key needs to be fetched.
     * @return itResTypeKey   : Key associated with the IT Resource Type.
     * @throws ContractorIDPPEventHandlerException
     */
    public long getITResourceTypeKey(String itResourceType) throws ContractorIDPPEventHandlerException {
        long itResTypeKey = -1;
        try {
            LOGGER("Enter OIMService#getITResourceTypeKey()");
            Map phAttributes = new HashMap();

            LOGGER("itResourceType " + itResourceType);

            phAttributes.put(Constants.IT_RESOURCE_TYPE_DEFINITION_SERVER_TYPE,
                             itResourceType);
            LOGGER("IT_RESOURCE_TYPE_DEFINITION_SERVER_TYPE  : " +
                   Constants.IT_RESOURCE_TYPE_DEFINITION_SERVER_TYPE);
            LOGGER("oimITResDefUtility :-  " + oimITResDefUtility);


            tcResultSet resultSet =
                oimITResDefUtility.getITResourceDefinition(phAttributes);

            if (resultSet != null && !resultSet.isEmpty()) {
                itResTypeKey =
                        resultSet.getLongValue(Constants.IT_RESOURCE_TYPE_DEFINITION_KEY);
            }

        } catch (tcColumnNotFoundException ex) {
            String reason =
                "tcColumnNotFoundException occurred when fetching key of the IT resource type " +
                itResourceType;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (tcAPIException ex) {
            String reason =
                "tcAPIException occurred when fetching key of the IT resource type " +
                itResourceType;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (Exception ex) {
            String reason =
                "Exception occurred when fetching key of the IT resource type " +
                itResourceType;
            throw new ContractorIDPPEventHandlerException(reason, ex);
        }
        LOGGER("Exit OIMService#getITResourceTypeKey()");
        return itResTypeKey;
    }


    /**
     * This method is used to get the Lookup values from the provided lookup code.
     * @param psLookupCode      : Lookup code defined for database in OIM environment.
     * @return lookupResultSet  : This is the resultSet containing the lookup values.
     * @throws ContractorIDPPEventHandlerException
     */
    public tcResultSet getLookupCodeInfo(String psLookupCode) throws ContractorIDPPEventHandlerException {
        tcResultSet lookupResultSet = null;
        //        String lookupCodeKey = null;
        try {
            lookupResultSet = lookupOperations.getLookupValues(psLookupCode);

            LOGGER("Enter OIMService#getLookupCodeInfo()");

            if (lookupResultSet != null && !lookupResultSet.isEmpty()) {
                LOGGER("resultSet is not empty");
            }
        } catch (tcAPIException ex) {
            String reason =
                "tcAPIException occurred while fetching lookup code key : ";
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (tcInvalidLookupException ex) {
            String reason =
                "tcInvalidLookupException occurred while fetching lookup code key : ";
            throw new ContractorIDPPEventHandlerException(reason, ex);
        } catch (Exception ex) {
            String reason =
                "Exception occurred while fetching lookup code key : ";
            throw new ContractorIDPPEventHandlerException(reason, ex);
        }
        LOGGER("Exit OIMService#getLookupCodeInfo()");
        return lookupResultSet;
    }

    public static void LOGGER(String msg) {
        System.out.println("LOGGER >> " + msg);
    }


}
